#include "RealmCharacterGroupPositioner.h"
#include "RealmCharacterObjectPositioner.h"
#include "RealmSceneView.h"
#include "RealmZoomAnimator.h"

using namespace std;

RealmCharacterGroupPositioner::RealmCharacterGroupPositioner(
									RealmSceneView* scene_view, HexSpiralCoordinate start_scene_coordinate,
									RealmSceneView* zone_view, HexSpiralCoordinate start_zone_coordinate,
									RealmSceneView* region_view, HexSpiralCoordinate start_region_coordinate,
									RealmSceneView* realm_world_view, HexSpiralCoordinate start_realm_coordinate
								) :
		RealmWorldAnimator(realm_world_view, start_realm_coordinate),
		RegionAnimator(region_view, start_region_coordinate),
		ZoneAnimator(zone_view, start_zone_coordinate),
		SceneAnimator(scene_view, start_scene_coordinate)

{
	HexSpiralCoordinate start_scene_character_coordinate(0), start_zone_character_coordinate(0), start_region_character_coordinate(0);

	using std::placeholders::_1;
	SceneAnimator.setMoveFinishEvent(std::bind(&RealmSceneView::moveFinished, scene_view, _1) );
	SceneAnimator.setMoveStartEvent(std::bind(&RealmSceneView::moveStarting, scene_view, _1) );

	ZoneAnimator.setMoveFinishEvent(std::bind(&RealmSceneView::moveFinished, zone_view, _1) );
	ZoneAnimator.setMoveStartEvent(std::bind(&RealmSceneView::moveStarting, zone_view, _1) );

	RegionAnimator.setMoveFinishEvent(std::bind(&RealmSceneView::moveFinished, region_view, _1) );
	RegionAnimator.setMoveStartEvent(std::bind(&RealmSceneView::moveStarting, region_view, _1) );

	RealmWorldAnimator.setMoveFinishEvent(std::bind(&RealmSceneView::moveFinished, realm_world_view, _1) );
	RealmWorldAnimator.setMoveStartEvent(std::bind(&RealmSceneView::moveStarting, realm_world_view, _1) );



	SceneZoomAnimator = scene_view->getZoomAnimator();
	ZoneZoomAnimator = zone_view->getZoomAnimator();
	RegionZoomAnimator = region_view->getZoomAnimator();

	region_view->addCameraAnimator(&RegionAnimator);
	zone_view->addCameraAnimator(&ZoneAnimator);
	realm_world_view->addCameraAnimator(&RealmWorldAnimator);

	ZoomMode = AT_REALM;

	for (int i = 0; i < 7; i++) {
		int scene_hex;
		if (i == 0)
			scene_hex = 0;
		else
			scene_hex = 20 + (i-1) * 3;
		start_scene_character_coordinate.setHexIndex(scene_hex);
		start_zone_character_coordinate.setHexIndex(i);
		CharacterPositioner[i] = new RealmCharacterObjectPositioner(this,
										scene_view, start_scene_character_coordinate,
										zone_view, start_zone_character_coordinate,
										region_view, start_region_character_coordinate);
	}
}

RealmCharacterGroupPositioner::~RealmCharacterGroupPositioner() {
	for (int i= 0; i < 7; i++) {
		delete CharacterPositioner[i];
	}
}


void RealmCharacterGroupPositioner::setScenePosition(HexSpiralCoordinate c) { SceneAnimator.setHexPosition(c); }
void RealmCharacterGroupPositioner::setZonePosition(HexSpiralCoordinate c) { ZoneAnimator.setHexPosition(c); }
void RealmCharacterGroupPositioner::setRegionPosition(HexSpiralCoordinate c) { RegionAnimator.setHexPosition(c); }


RealmZoomAnimator* RealmCharacterGroupPositioner::getZoomAnimator() {
	switch (ZoomMode) {
		case INTO_REGION:
		case OUT_REGION:
		case AT_REALM:
		case AT_REGION:
			return RegionZoomAnimator;

		case INTO_ZONE:
		case OUT_ZONE:
		case AT_ZONE:
			return ZoneZoomAnimator;

		case INTO_SCENE_PHASE1:
		case INTO_SCENE_PHASE2:
		case OUT_SCENE_PHASE1:
		case OUT_SCENE_PHASE2:
		case AT_SCENE:
			return SceneZoomAnimator;
	}
}

double RealmCharacterGroupPositioner::getZoomProgress() {
	RealmZoomAnimator* zoom_animator = getZoomAnimator();
	switch (ZoomMode) {
		case INTO_REGION:
		case OUT_REGION:

		case INTO_ZONE:
		case OUT_ZONE:

		case INTO_SCENE_PHASE1:
		case INTO_SCENE_PHASE2:
		case OUT_SCENE_PHASE1:
		case OUT_SCENE_PHASE2:
			return zoom_animator->getZoomProgress();

		default:
			return 1; // assume done (100%)
	}
}

void RealmCharacterGroupPositioner::startTransition(ZoomState new_mode)
{
	ZoomMode = new_mode;
	switch (ZoomMode) {
		case INTO_REGION:
		case OUT_REGION:
			SourceScreenLocation = RegionAnimator.getScreenLocation();
			TargetScreenLocation = RegionAnimator.getScreenLocation(true);
			break;

		case INTO_ZONE:
			SourceScreenLocation = RegionAnimator.getScreenLocation();
			TargetScreenLocation = ZoneAnimator.getScreenLocation(true);
			break;

		case OUT_ZONE:
			SourceScreenLocation = ZoneAnimator.getScreenLocation();
			TargetScreenLocation = RegionAnimator.getScreenLocation(true);
			break;


		case INTO_SCENE_PHASE1:
			SourceScreenLocation = ZoneAnimator.getScreenLocation();
			TargetScreenLocation = SceneAnimator.getScreenLocation(true);
			break;

		case INTO_SCENE_PHASE2:
			SourceScreenLocation = SceneAnimator.getScreenLocation();
			TargetScreenLocation = SceneAnimator.getScreenLocation(true);
			break;

		case OUT_SCENE_PHASE1:
			SourceScreenLocation = SceneAnimator.getScreenLocation();
			TargetScreenLocation = SceneAnimator.getScreenLocation(true);
			break;

		case OUT_SCENE_PHASE2:
			SourceScreenLocation = SceneAnimator.getScreenLocation();
			TargetScreenLocation = ZoneAnimator.getScreenLocation(true);
			break;

		default:
			return;  // skip CharacterPositioner calls below since they're only needed for state transition modes
	}

	for (int i= 0; i < 7; i++) {
		CharacterPositioner[i]->startTransition(new_mode);
	}
}

void RealmCharacterGroupPositioner::update() {
	RegionAnimator.update();
	ZoneAnimator.update();
	SceneAnimator.update();

	for (int i = 0; i < 7; i++) {
		CharacterPositioner[i]->update();
	}
}

double RealmCharacterGroupPositioner::getScaleFactor() {
	// return getZoomAnimator()->getZoomScale(1);
	return 1;
}

XYLocation RealmCharacterGroupPositioner::getScreenLocation() {

	switch (ZoomMode) {
		case AT_REALM:
		case AT_REGION:
			return RegionAnimator.getScreenLocation();
		case AT_ZONE:
			return ZoneAnimator.getScreenLocation();
		case AT_SCENE:
			return SceneAnimator.getScreenLocation();

		case INTO_REGION:
		case OUT_REGION:
		case INTO_ZONE:
		case OUT_ZONE:
		case INTO_SCENE_PHASE1:
		case INTO_SCENE_PHASE2:
		case OUT_SCENE_PHASE1:
		case OUT_SCENE_PHASE2:
			return XYLocation::findPointBetween(SourceScreenLocation, TargetScreenLocation, getZoomProgress() );
	}

}

